<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base\Models;

use Infomaker\Everyware\Support\Interfaces\Arrayable;
use Infomaker\Everyware\Support\Interfaces\Jsonable;
use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Fluent;
use Infomaker\Everyware\Support\Str;
use WP_Post;

/**
 * Post
 *
 * @property int    id
 * @property array  ancestors
 * @property int    post_parent
 * @property string post_title
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base\Models
 * @since   Infomaker\Everyware\Base\Models\Post 1.0.0
 */
class Post extends Fluent
{
    /**
     * @var string
     */
    protected static $type = 'post';

    /**
     * Defaults for fetching posts in Wordpress
     *
     * @var array
     */
    private static $defaultOptions = [
        'sort_order' => 'ASC',
        'sort_column' => 'post_title',
        'hierarchical' => true,
        'exclude' => '',
        'include' => '',
        'meta_key' => '',
        'meta_value' => '',
        'authors' => '',
        'child_of' => 0,
        'parent' => -1,
        'posts_per_page' => -1,
        'exclude_tree' => '',
        'number' => '',
        'offset' => 0,
        'post_status' => 'publish'
    ];

    public function __construct($post = [])
    {
        parent::__construct($this->getArrayableItems($post));
    }

    public function addMeta(string $key, $value)
    {
        if ($this->updateAllowed()) {
            return add_post_meta($this->id, $key, self::jsonEncodeWithWordpressWorkaround($value));
        }

        return false;
    }

    public function ancestors(): Collection
    {
        return Collection::make($this->ancestors)->map(function ($ancestor) {
            return static::createFromId($ancestor);
        });
    }

    public function children(array $options = []): Collection
    {
        return static::all(array_replace_recursive(['child_of' => $this->id], $options));
    }

    public function editUrl()
    {
        return get_edit_post_link($this->id);
    }

    public function getMeta(string $key = '', $default = [])
    {
        if (Str::isEmpty($key)) {
            return get_post_meta($this->id, $key, true);
        }

        $meta = get_post_meta($this->id, $key, true);

        $decodedMeta = json_decode($meta, true);

        if ($decodedMeta !== null) {
            return $decodedMeta;
        }

        return $meta ?: $default;
    }

    public function hasParent(): bool
    {
        return (int)$this->post_parent !== 0;
    }

    public function hierarchicalName(): string
    {
        $text = $this->post_title;
        $ancestors = $this->post_parent > 0 ? \count($this->ancestors()) : 0;

        return Str::prepend(str_repeat('— ', $ancestors), $text);
    }

    public function parent()
    {
        return $this->hasParent() ? static::createFromId($this->post_parent) : null;
    }

    public function permalink()
    {
        return static::getPermalink($this->id);
    }

    public function removeMeta(string $key)
    {
        if ($this->updateAllowed()) {
            return delete_post_meta($this->id, $key);
        }

        return false;
    }

    public function title(string $prefix = ''): string
    {
        $title = $this->post_title;

        return Str::notEmpty($prefix) ? Str::append($prefix, $title, ' - ') : $title;
    }

    public function updateAllowed(): bool
    {
        if (isset($_POST['post_type']) && $_POST['post_type'] === static::$type) {
            return current_user_can('edit_' . static::$type, $this->id);
        }

        return false;
    }

    public function updateMeta(string $key, $value)
    {
        if ($this->updateAllowed()) {
            return update_post_meta($this->id, $key, self::jsonEncodeWithWordpressWorkaround($value));
        }

        return false;
    }

    protected function getArrayableItems($items): array
    {
        if (\is_array($items)) {
            return $items;
        }
        if ($items instanceof WP_Post) {
            return array_change_key_case($items->to_array(), CASE_LOWER);
        }
        if ($items instanceof Arrayable) {
            return $items->toArray();
        }
        if ($items instanceof Jsonable) {
            return json_decode($items->toJson(), true);
        }

        return (array)$items;
    }

    /**
     * Retrieve all posts with optional filtering options
     *
     * @param array $options
     *
     * @uses  get_posts()
     * @return Collection
     */
    public static function all(array $options = []): Collection
    {
        return Collection::make(get_posts(static::setupOptions($options)))->map([static::class, 'createFromPost']);
    }

    public static function createFromId(int $id = null)
    {
        $newPost = get_post($id);
        if ($newPost !== null) {
            return new static($newPost);
        }

        return null;
    }

    public static function createFromPost(WP_Post $post)
    {
        return new static($post);
    }

    public static function createFromUrl($url)
    {
        return static::createFromId(url_to_postid($url));
    }

    public static function current()
    {
        return static::createFromId();
    }

    public static function getPermalink(int $post_id)
    {
        return get_permalink($post_id);
    }

    private static function setupOptions(array $options = []): array
    {
        return array_replace_recursive(static::$defaultOptions, ['post_type' => static::$type], $options);
    }

    /**
     * Encode variable to JSON string for saving as metadata, working around how Wordpress uses magic quotes.
     *
     * Without this, Wordpress will mess up JSON-encoded strings that contain linebreaks or single quotes.
     *
     * @param mixed $value
     *
     * @return string
     */
    private static function jsonEncodeWithWordpressWorkaround($value): string
    {
        // Counteract how wp_magic_quotes() has added slashes to form data.
        $value = stripslashes_deep($value);

        $value = json_encode($value, JSON_UNESCAPED_UNICODE);

        // Counteract how add_metadata() or update_metadata() is going to run wp_unslash() on the value before saving.
        $value = wp_slash($value);

        return $value;
    }

}
