<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Support\Str;

/**
 * Utilities
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\Utilities 1.0.0
 */
class Utilities
{

    /**
     * Function for fetching the correct js uri depending on if you are using the base- or child-theme's javascript
     *
     * @param string $file_name
     *
     * @return string
     */
    public static function getCorrectJsUrl(string $file_name = ''): string
    {
        return Str::finish(static::getStaticTemplateUri(), "/assets/js/{$file_name}");
    }

    /**
     * Return template uri for static assets, if prod return with cloudfront.
     *
     * @return string
     */
    public static function getStaticTemplateUri(): string
    {
        return static::getCdnUri(static::wpGetTemplateDirectoryUri());
    }

    /**
     * Retrieve the cdn version of chosen uri, if prod return with cloudfront
     *
     * @param string $uri
     *
     * @return string
     */
    public static function getCdnUri(string $uri = ''): string
    {
        $cdn_uri = env('CF_STATIC');
        $uri = empty($uri) ? static::wpGetStylesheetDirectoryUri() : $uri;

        if (null !== $cdn_uri && is_prod()) {
            return substr_replace($uri, $cdn_uri, 0, strpos($uri, '/wp-content/'));
        }

        return $uri;
    }

    /**
     * Function to retrieve current host
     *
     * @return mixed
     */
    public static function getCurrentHost()
    {
        return parse_url(site_url(), PHP_URL_HOST);
    }

    /**
     * Return the absolute path of the requested plugin
     *
     * @param string $plugin_dir name of the plugin dir
     *
     * @return string
     */
    public static function pluginPath(string $plugin_dir): string
    {
        return ABSPATH . 'wp-content/plugins/' . ltrim($plugin_dir, '/');
    }

    /**
     * Return path to plugin uri
     *
     *
     * @param string $plugin_dir name of the plugin
     *
     * @return mixed|string
     */
    public static function getStaticPluginUri(string $plugin_dir = '')
    {
        return static::getCdnUri(static::wpPluginsUrl($plugin_dir));
    }

    /**
     * Return theme uri for static assets, if prod return with cloud-front.
     *
     * @return string
     */
    public static function getStaticThemeUri(): string
    {

        return static::getCdnUri(static::wpGetStylesheetDirectoryUri());
    }

    /**
     * Retrieve the cdn uri for theme files, if prod return with cloudfront
     *
     * @param string $file_path
     *
     * @return string
     */
    public static function getCdnFile(string $file_path): string
    {
        return Str::finish(static::getCdnUri(), '/') . ltrim($file_path, '/\\');
    }

    /**
     * Determine if current theme is a child theme
     *
     * @return bool
     */
    public static function isChildTheme(): bool
    {
        return static::childThemePath() !== static::themePath();
    }

    /**
     * Retrieve path to current theme
     *
     * @param string $path
     *
     * @return string
     */
    public static function childThemePath(string $path = ''): string
    {
        return Str::finish(\defined('STYLESHEETPATH') ? STYLESHEETPATH : static::wpGetStylesheetDirectory(), "/$path");
    }

    /**
     * Retrieve path to current theme. May return parent theme if on child
     *
     * @param string $path
     *
     * @return string
     */
    public static function themePath(string $path = ''): string
    {
        return Str::finish(\defined('TEMPLATEPATH') ? TEMPLATEPATH : static::wpGetTemplateDirectory(), "/$path");
    }

    /*
     * Trigger and render 404 template, then exit.
     */
    public static function trigger404(): void
    {
        global $wp_query;
        $wp_query->set_404();

        status_header(404);
        nocache_headers();

        load_template(get_404_template());

        exit;
    }

    /**
     * ====================================================================
     * WORDPRESS FACADES
     * ====================================================================
     */

    public static function wpGetTemplateDirectory()
    {
        return get_template_directory();
    }

    public static function wpGetTemplateDirectoryUri()
    {
        return get_template_directory_uri();
    }

    public static function wpGetStylesheetDirectory()
    {
        return get_stylesheet_directory();
    }

    public static function wpGetStylesheetDirectoryUri()
    {
        return get_stylesheet_directory_uri();
    }

    public static function wpPluginsUrl(string $plugin_dir = '')
    {
        return plugins_url($plugin_dir);
    }
}
